@@ -23,32 +23,34 @@ elif __file__:
 
 
 # ----------------------外部可调用部分
-def chrome(version="auto", path=None, name="chromedriver", os_type=None) -> str:
+def chrome(version="auto", path=None, name="chromedriver", os_type=None, is_arm=False) -> str:
     """
     version: 三种参数 auto(当前版本-默认)  latest(最新版本)  x.x(指定版本,2级就够,太细的版本可能匹配不到)
     path: 可以是目录(指定放置驱动文件的目录),可以是文件(指定驱动文件的路径)
     name: 下载的驱动文件名称(下载的文件名,除非镜像源的文件改名了,否则不要改动)
-    os_type: 默认自动检测,指定只能三选一['linux', 'win', 'mac']
+    os_type: 默认自动,手动填入系统信息,e.g. "win64" , "linux32-arm", "mac64 arm"
+    is_arm: 默认自动,是否ARM架构,匹配对应 arm/aarch64 驱动
     """
     # 判断版本
     browser_type = 'chrome'
-    lst_name, browser_version = __pre(browser_type, name, version, os_type)
+    dict_sys, browser_version = __pre(browser_type, name, version, os_type, is_arm)
     if not browser_version:
         return ''
     # 返回浏览器路径
-    return __handle(path, lst_name, browser_type, browser_version, browser_version)
+    return __handle(path, dict_sys, browser_type, browser_version, browser_version)
 
 
-def firefox(version="auto", path=None, name="geckodriver", os_type=None) -> str:
+def firefox(version="auto", path=None, name="geckodriver", os_type=None, is_arm=False) -> str:
     """
     version: 三种参数 auto(当前版本-默认)  latest(最新版本)  x.x(指定版本,2级就够,太细的版本可能匹配不到)
     path: 可以是目录(指定放置驱动文件的目录),可以是文件(指定驱动文件的路径)
     name: 下载的驱动文件名称(下载的文件名,除非镜像源的文件改名了,否则不要改动)
-    os_type: 默认自动检测,指定只能三选一['linux', 'win', 'mac']
+    os_type: 默认自动,手动填入系统信息,e.g. "win64" , "linux32-arm", "mac64 arm"
+    is_arm: 默认自动,是否ARM架构,匹配对应 arm/aarch64 驱动
     """
     # 判断版本
     browser_type = 'firefox'
-    lst_name, browser_version = __pre(browser_type, name, version, os_type)
+    dict_sys, browser_version = __pre(browser_type, name, version, os_type, is_arm)
     if not browser_version:
         return ''
     elif browser_version == 'latest':
@@ -70,35 +72,37 @@ def firefox(version="auto", path=None, name="geckodriver", os_type=None) -> str:
                 break
             else:
                 version = v[1]
-    return __handle(path, lst_name, browser_type, version, browser_version)
+    return __handle(path, dict_sys, browser_type, version, browser_version)
 
 
-def edge(version="auto", path=None, name="edgedriver", os_type=None) -> str:
+def edge(version="auto", path=None, name="edgedriver", os_type=None, is_arm=False) -> str:
     """
     version: 三种参数 auto(当前版本-默认)  latest(最新版本)  x.x(指定版本,2级就够,太细的版本可能匹配不到)
     path: 可以是目录(指定放置驱动文件的目录),可以是文件(指定驱动文件的路径)
     name: 下载的驱动文件名称(下载的文件名,除非镜像源的文件改名了,否则不要改动)
-    os_type: 默认自动检测,指定只能三选一['linux', 'win', 'mac']
+    os_type: 默认自动,手动填入系统信息,e.g. "win64" , "linux32-arm", "mac64 arm"
+    is_arm: 默认自动,是否ARM架构,匹配对应 arm/aarch64 驱动
     """
     # 判断版本
     browser_type = 'edge'
-    lst_name, browser_version = __pre(browser_type, name, version, os_type)
+    dict_sys, browser_version = __pre(browser_type, name, version, os_type, is_arm)
     version = browser_version
     if not version:
         return ''
-    return __handle(path, lst_name, browser_type, version, browser_version)
+    return __handle(path, dict_sys, browser_type, version, browser_version)
 
 
-def ie(version="auto", path=None, name="IEDriverServer", os_type=None) -> str:
+def ie(version="auto", path=None, name="IEDriverServer", os_type=None, is_arm=False) -> str:
     """
     version: 三种参数 auto(当前版本-默认)  latest(最新版本)  x.x(指定版本,2级就够,太细的版本可能匹配不到)
     path: 可以是目录(指定放置驱动文件的目录),可以是文件(指定驱动文件的路径)
     name: 下载的驱动文件名称(下载的文件名,除非镜像源的文件改名了,否则不要改动)
-    os_type: 默认自动检测,指定只能三选一['linux', 'win', 'mac']
+    os_type: 默认自动,手动填入系统信息,e.g. "win64" , "linux32-arm", "mac64 arm"
+    is_arm: 默认自动,是否ARM架构,匹配对应 arm/aarch64 驱动
     """
     # 判断版本
     browser_type = 'ie'
-    lst_name, browser_version = __pre(browser_type, name, version, os_type)
+    dict_sys, browser_version = __pre(browser_type, name, version, os_type, is_arm)
 
     if not browser_version:
         return ''
@@ -109,14 +113,14 @@ def ie(version="auto", path=None, name="IEDriverServer", os_type=None) -> str:
         except:
             version = 'latest'
     # 检查缓存的是否有对应版本driver
-    return __handle(path, lst_name, browser_type, version, browser_version)
+    return __handle(path, dict_sys, browser_type, version, browser_version)
 
 
-def clear_cache(path=None) -> None:
+def clear(path=None) -> None:
     """
-    path: 指定清除的目录(会删除目录下所有文件)
+    path: 清除缓存(删除保存的所有驱动文件)
     清除驱动缓存,清空默认目录 _webdriver 下所有内容
-    (适用驱动文件损坏或时间太久累积了较多无用驱动)
+    (适用驱动文件损坏或想移除已下载的驱动,驱动默认每种浏览器保留1个冗余版本)
     """
     if path:
         if not Path(path).is_dir():
@@ -143,59 +147,75 @@ def clear_cache(path=None) -> None:
 
 
 # ----------------------处理调用
-def __handle(path, lst_name, browser_type, version, browser_version):
+def __handle(path, dict_sys, browser_type, version, browser_version):
     """browser_version / version  for firefox 浏览器版本和下载链接版本不一致"""
     # 检查缓存的是否有对应版本driver
-    path_driver = __check_file(path, lst_name, browser_type, browser_version)
+    path_driver = __check_file(path, dict_sys, browser_type, browser_version)
     if path_driver.is_file():  # 如果检测文件已存在,返回路径
         print(f'--> 驱动路径 {browser_type} : {path_driver}')
         return str(path_driver)
 
     # 获取下载链接
-    file_url = __func_select(browser_type, lst_name, version)
+    file_url = __func_select(browser_type, dict_sys, version)
     if not file_url:
         return ''
+    # 删除冗余文件
+    sys_type = dict_sys['os'] + '64' if dict_sys['is_64'] else dict_sys['os'] + '32'
+    __clear_old_version(dir_path, sys_type)
     # 根据下载连接,下载解压文件
     result = __save_file(path_driver, file_url)
+    # 返回路径
     print(f'--> 驱动路径 {browser_type} : {result}')
-    return str(result)
+    return str(result.absolute())
 
 
 # ----------------------获取浏览器版本部分
-def __pre(browser_type, name, version, os_type) -> tuple:
+def __pre(browser_type, name, version, os_type, is_arm) -> tuple:
     """获取[平台,位数,文件名], 获取版本号"""
-    # 判断 平台, name 拆成 list, 用于判断 最终文件名是否包含 name 所有元素
-    lst_name = []
-    # 平台
-    if os_type and os_type.lower() in ['linux', 'win', 'mac']:
-        lst_name.append(os_type.lower())
-    else:
-        pl = sys.platform
-        if pl == "linux" or pl == "linux2":
-            lst_name.append('linux')
+    dict_sys = {'is_arm': False, }
+    # 根据手动填的 os_type ,拆分信息
+    if os_type:
+        os_type = os_type.lower()
+        for _ in ['win', 'linux', 'mac']:
+            if _ in os_type:
+                dict_sys['os'] = _
+                break
+        dict_sys['is_64'] = True if '64' in os_type else False
+        dict_sys['is_arm'] = True if 'arm' in os_type or 'arch' in os_type or is_arm else False
+    # 补全 os_type 缺少的信息
+    if not dict_sys.get('os'):  # 系统平台
+        pl = sys.platform.lower()
+        if 'linux' in pl:
+            dict_sys['os'] = 'linux'
         elif pl == "darwin":
-            lst_name.append('mac')
-        elif pl == "win32":
-            lst_name.append('win')
+            dict_sys['os'] = 'mac'
+        elif 'win' in pl:
+            dict_sys['os'] = 'win'
         else:
             print(f'警告,未知平台{pl},将使用win平台(Warning, unknown platform {pl}, will use Win)')
-    pl = lst_name[0]
-    # 位数
-    if platform.machine().endswith("64"):
-        lst_name.append('64')
-    else:
-        lst_name.append('32')
-    lst_name.append(name)
+    if not dict_sys.get('is_64'):  # 位数
+        dict_sys['is_64'] = True if platform.machine().endswith("64") else False
+    # mac 判断 is_arm(is_m1)
+    if dict_sys['os'] == 'mac' and not dict_sys.get('is_arm'):  # mac_m1
+        try:
+            info = subprocess.check_output(['sysctl', '-n', 'machdep.cpu.brand_string']).decode('utf-8')
+            dict_sys['is_arm'] = True if info == 'Apple M1\n' else False
+        except:
+            pass
+    # 通用 判断是否arm
+    if not dict_sys.get('is_arm'):  # arm/aarch
+        if 'aarch64' in ''.join(platform.uname()):
+            dict_sys['is_arm'] = True
+    pl = dict_sys['os']
     # 判断当前浏览器的版本(默认 auto), latest最新的, x.x.x 指定具体版本
     if version == 'auto':
         version = __get_browser_version_from_os(browser_type, pl)
-
+    dict_sys['file_name'] = name
     if version:
         print(f'--> 当前浏览器版本 {browser_type} : {version}')
     else:
         print(f'--> 获取版本失败 {browser_type} : 请指定具体版本或改成latest')
-
-    return lst_name, version
+    return dict_sys, version
 
 
 def __get_browser_version_from_os(browser_type, pl) -> str:
@@ -338,7 +358,7 @@ def __get_browser_version_from_os(browser_type, pl) -> str:
         version = __get_browser_version_from_os_by_powershell(cmd_mapping, pattern, pl)
     if not version:
         print(
-            f'!! 未获取到浏览器版本号,请指定 DriverManage.{browser_type}(version="latest") 或者 DriverManage.{browser_type}(version="x.x")')
+            f'!! 未获取到浏览器版本号,请指定 ez_webdriver.{browser_type}(version="latest") 或者 ez_webdriver.{browser_type}(version="x.x")')
     return version
 
 
@@ -386,7 +406,7 @@ def __get_browser_version_from_os_by_reg(cmd_mapping_win_reg, pattern) -> str:
 
 
 # ----------------------选择版本部分
-def __func_select(driver_type, lst_name, version) -> str:
+def __func_select(driver_type, dict_sys, version) -> str:
     """
     1.循环镜像源,找能用的
     2.选择对应获取链接的函数, 获取下载链接
@@ -417,13 +437,13 @@ def __func_select(driver_type, lst_name, version) -> str:
     file_url = None
     for site in sites:  # 判断 镜像源类型,执行对应函数
         if site[0] == 'npmmirror_json':
-            file_url = __by_npmmirror_json(site[1], lst_name, version)
+            file_url = __by_npmmirror_json(site[1], dict_sys, version)
         elif site[0] == 'googleapis_xml':
-            file_url = __by_googleapis_xml(site[1], lst_name, version)
+            file_url = __by_googleapis_xml(site[1], dict_sys, version)
         elif site[0] == 'githubapi_json':
-            file_url = __by_githubapi_json(site[1], lst_name, version)
+            file_url = __by_githubapi_json(site[1], dict_sys, version)
         elif site[0] == 'microsoft_html':
-            file_url = __by_microsoft_html(site[1], lst_name, version)
+            file_url = __by_microsoft_html(site[1], dict_sys, version)
         else:
             print(f'xx 错误,未知镜像源类型{site[0]}')
         if file_url:
@@ -465,29 +485,36 @@ def __version_select(d_version, version) -> list:
     return lst_ver  # 若最新版本无文件,往前找20个版本
 
 
-def __file_select(s, dir_url, lst_name) -> str:
+def __file_select(s, dir_url, dict_sys) -> str:
     f_url_default = ''
     # 根据文件名(仅需平台标识), 获取要下载的文件链接
     lst_file = json.loads(s.get(dir_url).text)
     for file in lst_file:
-        f_name = file['name']
+        f_name = file['name'].lower().replace('-', '_')
         f_url = file['url']
-        if f_name == lst_name[2] or f_name == '.'.join(f_name.split('.')[:-1]):  # 用户自定义文件名的情况
+        if f_name == dict_sys['file_name'] or f_name == '.'.join(f_name.split('.')[:-1]):  # 用户自定义文件名的情况
             return f_url
-        if lst_name[2] == 'IEDriverServer':  # ie只有win,64位名称不带win
-            if lst_name[1] in f_name:
+        # ie单独处理, 只有win,64位名称不带win
+        if dict_sys['file_name'] == 'IEDriverServer':
+            if dict_sys['is_64'] and '64' in f_name:
+                return f_url
+            elif not dict_sys['is_64'] and '64' not in f_name and '32' in f_name:
                 return f_url
 
-        if lst_name[0] in f_name and lst_name[2] in f_name:
-            if lst_name[1] in f_name:
+        # 其他浏览器判断方法  平台和文件名 -> arm -> m1 -> 位数
+        if dict_sys['os'] in f_name and dict_sys['file_name'] in f_name:
+            if dict_sys['is_arm'] and ('arm' not in f_name and 'arch' not in f_name and '_m1' not in f_name):
+                continue
+            elif not dict_sys['is_arm'] and ('arm' in f_name or 'arch' in f_name or '_m1' in f_name):
+                continue
+            if dict_sys['is_64'] and '64' in f_name:  # firefox  mac 32位文件无32标识
                 return f_url
-            f_url_default = f_url
-
+            f_url_default = f_url  # 保底措施
     return f_url_default
 
 
 # ----------------------处理镜像的url,获取下载地址
-def __by_npmmirror_json(url, lst_name, version) -> str:
+def __by_npmmirror_json(url, dict_sys, version) -> str:
     """
     处理 npmmirror url
     默认获取 用户电脑上匹配版本
@@ -512,7 +539,7 @@ def __by_npmmirror_json(url, lst_name, version) -> str:
         return ''
     for dir_url in lst_ver:  # 循环 20个版本,新->旧 ,找能用的
         # 根据文件名(仅需平台标识),判断要下载的文件
-        file_url = __file_select(s, dir_url, lst_name)
+        file_url = __file_select(s, dir_url, dict_sys)
         if file_url:
             return file_url
     return ''
@@ -635,23 +662,36 @@ def __by_microsoft_html(url, lst_name, version) -> str:
 
 
 # ----------------------保存驱动文件部分(返回Path路径)
-def __check_file(path, lst_name, browser_type, version) -> Path:
+def __check_file(path, dict_sys, browser_type, version) -> Path:
     """"检查目录下是否有对应浏览器版本的驱动文件"""
-
     dir_path = Path(path) if path else Path(dir_pre / '_webdriver')
     dir_path.mkdir(parents=True, exist_ok=True)
-
-    _ = version + '-' + '-'.join(lst_name[:2])
+    sys_bit = '64' if dict_sys['is_64'] else '32'
+    sys_type = dict_sys['os'] + sys_bit
+    _ = sys_type + '-' + version
     dir_file = dir_path / browser_type / _
     if dir_file.is_dir():
         for f in dir_file.glob('*.*'):
-            if f.stem == lst_name[2]:
+            if f.stem == dict_sys['file_name']:
                 return f
 
     dir_file.mkdir(parents=True, exist_ok=True)  # 创建对应目录
     return dir_file
 
 
+def __clear_old_version(dir_path,sys_type):
+    """下载文件前检查,存在2个以上就只留一个"""
+    try:
+        lst_dir = list(dir_path.glob(sys_type+'.*'))
+        if len(lst_dir)>1:
+            for i in lst_dir[1:].sort(reverse=True):
+                os.remove(i)
+    except:
+        pass
+
+
+
+
 def __save_file(path_driver, file_url) -> Union[Path, str]:
     """下载保存文件"""
     try:
@@ -666,7 +706,6 @@ def __save_file(path_driver, file_url) -> Union[Path, str]:
             print("\r%s %.2f%%" % (show_progress, percent * 100), end='')
 
         # 断点续传
-
         temp_size = file_path.stat().st_size if file_path.exists() else 0
         s.headers.update({'Range': 'bytes=%d-' % temp_size, })
         res_left = s.get(file_url, stream=True)
@@ -695,7 +734,8 @@ def __save_file(path_driver, file_url) -> Union[Path, str]:
         print('xx 下载驱动文件失败(download failed)', err)
         return ''
     print('下载完成(finished)')
-    return path_driver
+    # 获取解压文件夹下第一个文件
+    return path_driver.glob('*.*').__next__()
 
 
 # ----------------------获取cmd结果,获取requests的session
@@ -718,5 +758,3 @@ def __session() -> requests.session():
     s.headers.update(headers)
     s.trust_env = False  # 禁用代理
     return s
-
-
